#ifndef __CAMERA_HPP
#define __CAMERA_HPP

#include <SophiaFramework.hpp>


#define PICTURE_WIDTH   240     // Width of image taken with camera
#define PICTURE_HEIGHT  320     // Height of image taken with camera


SFMTYPEDEFCLASS(Camera)

class Camera {

SFMSEALCOPY(Camera)

public:
    enum StatusEnum {           // Camera status
        STATUS_PREVIEW,         // Preview mode
        STATUS_FRAME,           // New frame of the preview was drawn
        STATUS_RECORD,          // Start of snapshot mode
        STATUS_ENCODE           // Completion of photography in snapshot mode
    };

private:

    typedef Void (*NotifySPP)(StatusEnum status, SFCError error, VoidPtr reference);

private:

    SFBCameraSmp _camera;       // ICamera interface
    StatusEnum _status;         // Camera status
    SFBBitmapSmp _bitmap;       // Photo image in bitmap
    SFXRectangle _rect;         // Display area of photo image on screen
    AEEMediaData _mediaData;    // Media data
    ACharPtr _mimeType;         // MIME TYPE
    Bool _previewable;          // Whether it is a possible preview now or not
    Bool _recordable;           // Whether photography is available in snapshot mode or not
    NotifySPP _spp;             // Camera callback function
    VoidPtr _reference;         // Reference data for the above callback function

    Void AfterSnapshot(Void);   // Processing after snapshot picture is taken

public:

    Camera(Void);                               // Constructor
    virtual ~Camera(Void);                      // Destructor
    SFCError Initialize(SFXRectangleConstRef rect, NotifySPP spp, VoidPtr reference);   // Camera initialization
    SFCError Preview(Void);                     // Beginning of Preview mode
    SFCError Record(Void);                      // Beginning of Snapshot mode
    Void Suspend(Void);                         // Suspend camera processing
    SFCError Resume(Void);                      // Resume camera processing
    Void Terminate(Void);                       // Release source
    SFBBitmapSmpRef GetBitmap(Void) {           // Get photo image in bitmap
        return _bitmap;
    }
    Bool IsPreviewable(Void) const {            // Whether it is a possible preview now
        return _previewable;
    }
    static Void OnCameraSHP(VoidPtr reference, AEECameraNotify* notify);    // Callback handler (dummy)
    Void OnCamera(AEECameraNotify *notify);                                 // Callback handler (body)
    SInt16 GetBrightness(Void);                                             // Get the current brightness of camera
    Bool GetBrightnessRange(SInt16Ptr minimum, SInt16Ptr maximum);          // Get the range of brightness available for camera
    Void SetBrightness(SInt16 value);                                       // Set the brightness of camera
    SInt16 GetZoom(Void);                                                   // Get the current zoom value
    Bool GetZoomRange(SInt16Ptr mininum, SInt16Ptr maximum);                // Get the range of zoom value available for camera
    Void SetZoom(SInt16 value);                                             // Set the zoom value
};

#endif
